/*4:*/
#line 55 "eigen.w"

/*3:*/
#line 43 "eigen.w"

 /* 
 Linear Calculation Library 
 
 Copyright (C) 2006: M Kudo & A. Tanaka
 
 Ver 1.0: 29 Sept 2006
 - This file is automatically generated from eigen.w. 
   See it for encoding and libmatrix.tex for documentation. 
 */

/*:3*/
#line 56 "eigen.w"

/*15:*/
#line 339 "eigen.w"

#include <iostream>  
#include <math.h>  
#include "matrix.h" 


extern YESNO do_orthonormal;

template<class T> 
inline const T&max(const T&a,const T&b)
{return a> b?a:b;}

template<class T> 
inline T abs(const T&a)
{return a> 0.0?a:-a;}

/*:15*/
#line 57 "eigen.w"

/*16:*/
#line 356 "eigen.w"

int Jacobi(const Matrix&X);
const Real EPS= 1.e-10;

const Real EPS2= EPS;

/*:16*/
#line 58 "eigen.w"

/*6:*/
#line 107 "eigen.w"


int Jacobi(const Matrix&X,Matrix&Z,Matrix&U)



{

#ifdef efficiency_than_math
cout<<"In Jacobi; * Efficient is preferred !"<<endl;
#else
cout<<"In Jacobi; * Mathimatics is preferred !"<<endl;
#endif

Matrix Y= X;
int nrows= Y.Nrows();
int ncols= Y.Ncols();
int p,q;

IdentityMatrix I(nrows);
Matrix P(nrows,ncols);
IdentityMatrix V(nrows);

Real theta;
Real c,s,t;

Real S,S0;

#ifdef secure
/*7:*/
#line 198 "eigen.w"

{
if(Y.Ncols()!=Y.Nrows())
{
cerr<<"*E* This matrix is not squared !"<<endl;
return(-1);
}

for(int i= 0;i<nrows;i++)
for(int j= i+1;j<ncols;j++)
if(Y(i,j)!=Y(j,i))
{
cerr<<"*E* This matrix is not symmetric at ("<<i<<","<<j<<")"<<endl;
return(-1);
}
}

/*:7*/
#line 136 "eigen.w"
;
#endif

/*13:*/
#line 286 "eigen.w"

{
S= 0.0;
for(int i= 0;i<nrows;i++)
for(int j= i+1;j<ncols;j++)
S+= 2.0*Y(i,j)*Y(i,j);
}

/*:13*/
#line 139 "eigen.w"
;
#ifdef verbose
cout<<"Current error = "<<S<<endl;
#endif
S0= S;

int loop= 0;
while(/*8:*/
#line 216 "eigen.w"

(S> EPS*S0)?1:0

/*:8*/
#line 146 "eigen.w"
)
{
#ifdef verbose
Y.Print("Currect Y");
#endif

/*9:*/
#line 220 "eigen.w"

{
Real vmax= -1.e33;
p= q= (-1);

for(int i= 0;i<nrows;i++)
for(int j= i+1;j<ncols;j++)
if(abs(Y(i,j))> vmax)
{
vmax= abs(Y(i,j));
p= i;
q= j;
}


}

/*:9*/
#line 152 "eigen.w"


#ifdef verbose
cout<<"Max value of "<<Y(p,q)<<" at ("<<p<<","<<q<<")"<<endl;
#endif
/*10:*/
#line 238 "eigen.w"

{
theta= (Y(q,q)-Y(p,p))/(2.0*Y(p,q));
t= 1.0/(abs(theta)+sqrt(theta*theta+1.0));
t= (theta>=0.0)?t:-t;
c= 1.0/sqrt(t*t+1.0);
s= t*c;


P= I;
P(p,p)= P(q,q)= c;
P(p,q)= s;
P(q,p)= -s;
}

/*:10*/
#line 157 "eigen.w"


#ifdef verbose
P.Print("Rotate P");
#endif

/*12:*/
#line 280 "eigen.w"

{
S-= 2*Y(p,q)*Y(p,q);
}

/*:12*/
#line 163 "eigen.w"

#ifdef verbose
cout<<loop<<"th Current error = "<<S<<" for convergence "<<S0*EPS<<endl;
#endif

/*11:*/
#line 254 "eigen.w"

{
#ifdef efficiency_than_math
Y(p,p)-= t*Y(p,q);
Y(q,q)+= t*Y(p,q);
Real u= s/(1.0+c);

Real Yrp,Yrq;
for(int r= 0;r<ncols;r++)
{
if(r==p||r==q)
continue;
Yrp= Y(r,p);
Yrq= Y(r,q);
Y(p,r)= Y(r,p)-= (s*(Yrq+u*Yrp));
Y(q,r)= Y(r,q)+= (s*(Yrp-u*Yrq));
}
Y(p,q)= Y(q,p)= 0.0;

#else
Y= P.t()*Y*P;
#endif

}

/*:11*/
#line 168 "eigen.w"


#ifdef verbose
Y.Print("Rotated Y");
cout<<"* trace = "<<Y.Tr()<<endl;
#endif

V*= P;

#ifdef verbose
V.Print("Multiplied P");
#endif

loop++;
}




/*14:*/
#line 295 "eigen.w"

{
int*reorder= new int[nrows];
Real*value= new Real[nrows];
for(int i= 0;i<nrows;i++)
{
reorder[i]= i;
value[i]= Y(i,i);
}

Real vtmp;
int itmp;

for(int i= 0;i<nrows;i++)
for(int j= nrows-1;j> i;j--)
{
if(value[j-1]<value[j])
{
vtmp= value[j];
value[j]= value[j-1];
value[j-1]= vtmp;

itmp= reorder[j];
reorder[j]= reorder[j-1];
reorder[j-1]= itmp;
}
}

for(int i= 0;i<nrows;i++)
{

Z(i,i)= Y(reorder[i],reorder[i]);
U.SetCols(i,i,V.Cols(reorder[i],reorder[i]));
}

#ifdef verbose   
(U.t()*U).Print("U.t()*U");
#endif

}



/*:14*/
#line 187 "eigen.w"


#ifdef verbose
Z.Print("Eigen Values in decresing order");
U.Print("Eigen Vectors as column vectors in decresaing order of the associated eigen values");
#endif

return 0;
}

/*:6*/
#line 59 "eigen.w"

/*18:*/
#line 384 "eigen.w"

void MaximumEigenMethod(const Matrix&X,Matrix&Z,Matrix&U,int&rank,Real&ratio)


{


#ifdef efficiency_than_math
cout<<"In MaximumEigenMethod; * Efficient is preferred !"<<endl;
cout<<"X.rows = "<<X.Nrows()<<" X.cols = "<<X.Ncols()<<endl;
#else
cout<<"In MaximumEigenMethod; * Mathimatics is preferred !"<<endl;
cout<<"X.rows = "<<X.Nrows()<<" X.cols = "<<X.Ncols()<<endl;
#endif


/*19:*/
#line 475 "eigen.w"

Matrix Y= X;
int nrows= Y.Nrows();
int ncols= Y.Ncols();
Matrix TmpM;
ColumnVector Tmpv(nrows);


U= 0.0*U;
Z= 0.0*Z;

int required_rank= nrows;
Real required_ratio= 1.0;

if(rank> 0&&rank<=nrows)
required_rank= rank;

if(ratio> 0.0&&ratio<=1.0)
required_ratio= ratio;

cout<<"required_rank = "<<required_rank<<" required_ratio = "<<
required_ratio<<endl;

Real trace= 0.0;
Real norm= 0.0;
Real root= 0.0;
Real error= 0.0;

/*:19*/
#line 400 "eigen.w"


#ifdef verbose 
Y.Print("Y");
#endif

Real full_trace= Y.Tr();
int found_rank= 0;
Real attained_ratio= 0.0;
int loop= 0;

ColumnVector v(nrows);
ColumnVector vp(nrows);
IdentityMatrix V(nrows);
ColumnVector One(nrows,1.0);

/*20:*/
#line 504 "eigen.w"

RandomMatrix R(nrows,1);
v<<R;
norm= v.Norm();
v/= norm;

/*:20*/
#line 416 "eigen.w"


do{
loop= 0;

/*30:*/
#line 573 "eigen.w"

{





ColumnVector u(nrows);
ColumnVector uu(nrows);

if(found_rank> 0)
{
V.SetCols(found_rank-1,found_rank-1,v);
u= U.Cols(found_rank-1,found_rank-1);
for(int i= found_rank;i<nrows;i++)
{
uu= u*u.t()*V.Cols(i,i);
u= V.Cols(i,i)-uu;
u/= u.Norm();
V.SetCols(i,i,u);
}

}

v= V.Cols(found_rank,found_rank);
v/= v.Norm();


}

/*:30*/
#line 421 "eigen.w"



do{

/*22:*/
#line 517 "eigen.w"

Tmpv= Y*v;
vp= Tmpv;

root= vp.Norm();

/*:22*/
#line 426 "eigen.w"


if(/*23:*/
#line 524 "eigen.w"

root<EPS

/*:23*/
#line 428 "eigen.w"
)
{

/*24:*/
#line 528 "eigen.w"

RandomMatrix R(nrows,1);
v= R;
norm= v.Norm();
v/= norm;
Tmpv= Y*v;
vp= Tmpv;

root= vp.Norm();

/*:24*/
#line 431 "eigen.w"

}

/*25:*/
#line 539 "eigen.w"

vp/= root;
error= (vp-v).Norm();
v= vp;

/*:25*/
#line 434 "eigen.w"


loop++;




#ifdef verbose
cout<<"For "<<found_rank<<"th e.v. in loop"<<loop
<<" th current error = "<<error<<endl;
#endif

}while(/*26:*/
#line 545 "eigen.w"

error> EPS2

/*:26*/
#line 446 "eigen.w"
);

trace+= root;
attained_ratio= trace/full_trace;

#ifdef verbose 
/*27:*/
#line 549 "eigen.w"

cout<<"---------------------"<<endl;
cout<<found_rank<<" th eigen value"<<endl;
cout<<"Eigen values = "<<root<<endl;
cout<<"Trace = "<<trace<<" for full_trace = "<<full_trace<<" ("<<
attained_ratio*100.0<<"%)"<<endl;
cout<<"--------------------"<<endl;

/*:27*/
#line 452 "eigen.w"

#endif


/*28:*/
#line 558 "eigen.w"


Z(found_rank,found_rank)= root;
U.SetCols(found_rank,found_rank,v);

found_rank++;

/*:28*/
#line 456 "eigen.w"


/*29:*/
#line 566 "eigen.w"

TmpM= v*v.t();
TmpM*= root;
Y-= TmpM;


/*:29*/
#line 458 "eigen.w"


cout<<found_rank<<"th e.v. ( "<<root<<" ) was found ! in loop "<<loop<<
" with cont. ratio "<<attained_ratio<<endl;

}while(/*32:*/
#line 631 "eigen.w"

attained_ratio<(required_ratio-EPS)&&found_rank<required_rank

/*:32*/
#line 463 "eigen.w"
);

rank= found_rank;
ratio= attained_ratio;
if(rank<nrows&&do_orthonormal==YES)
/*31:*/
#line 604 "eigen.w"

{

cout<<"In Orthonomalization of eigen vectors for the ramining "<<nrows-found_rank<<endl;



ColumnVector u(nrows);
ColumnVector uu(nrows);

for(int j= found_rank;j<nrows;j++)
{
u= V.Cols(j,j);
for(int i= 0;i<j;i++)
{
uu= U.Cols(i,i);
u-= uu*uu.t()*u;
}
u/= u.Norm();
U.SetCols(j,j,u);
}



}

/*:31*/
#line 468 "eigen.w"
;



}

/*:18*/
#line 60 "eigen.w"


/*:4*/
