/*4:*/
#line 56 "gauss.w"

/*3:*/
#line 44 "gauss.w"

 /* 
 Linear Calculation Library 
 
 Copyright (C) 2006: M Kudo & A. Tanaka
 
 Ver 1.0: 29 Sept 2006
 - This file is automatically generated from gauss.w. 
   See gauss.tex for understanding its contents. 
 */

/*:3*/
#line 57 "gauss.w"

/*11:*/
#line 260 "gauss.w"

#include <iostream>  
#include <math.h>  
#include "matrix.h" 

template<class T> 
inline const T&max(const T&a,const T&b)
{return a> b?a:b;}

template<class T> 
inline T abs(const T&a)
{return a> 0.0?a:-a;}

const Real EPS= 1.e-10;

/*:11*/
#line 58 "gauss.w"

/*6:*/
#line 92 "gauss.w"

Matrix GaussJordan(const Matrix&X)
{

#ifdef verbose

#ifdef efficiency_than_math
cout<<"In GaussJordan; * Efficient is preferred !"<<endl;
#else
cout<<"In GaussJordan; * Mathimatics is preferred !"<<endl;
#endif

#endif

Matrix Y= X;
int nrows= Y.Nrows();
int ncols= Y.Ncols();

#ifdef secure
if(nrows!=ncols)
{
cerr<<"*Error* This matrix is not square: nrows= "<<nrows<<" ncols = "<<
ncols<<endl;
Y.Print("Current status of matrix Y");
return Y;
}
#endif

const IdentityMatrix I(nrows);
Matrix S= I;

int j;

for(int k= 0;k<ncols;k++)
{
#ifdef verbose
Y.Print("(1) Y");S.Print("(1) S");
#endif
Real vmax;
/*7:*/
#line 164 "gauss.w"

{
vmax= abs(Y(k,k));
j= k;

for(int i= k+1;i<nrows;i++)
if(abs(Y(i,k))> vmax)
{
vmax= abs(Y(i,k));
j= i;
}

}

/*:7*/
#line 131 "gauss.w"

#ifdef verbose
Y.Print("(2) Y");S.Print("(2) S");
#endif


if(vmax<EPS)
{
cerr<<"*Error* This matrix is sigular."<<endl;
Y.Print("Current status of matrix Y");
return Y;
}

if(j> k)
/*8:*/
#line 179 "gauss.w"

{

#ifdef efficiency_than_math
Real v;
for(int i= 0;i<ncols;i++)
{
v= Y(k,i);
Y(k,i)= Y(j,i);
Y(j,i)= v;

v= S(k,i);
S(k,i)= S(j,i);
S(j,i)= v;
}
#else
FundOpeMatrixTypeII F2(ncols,k,j);
#ifdef verbose
F2.Print("F2");
#endif
Y= F2*Y;
S= F2*S;
#endif

}

/*:8*/
#line 145 "gauss.w"


#ifdef verbose
Y.Print("(3) Y");S.Print("(3) S");
#endif
/*9:*/
#line 206 "gauss.w"

{
#ifdef efficiency_than_math
Real v= Y(k,k);
for(int i= 0;i<ncols;i++)
{
Y(k,i)/= v;
S(k,i)/= v;
}
#else
FundOpeMatrixTypeI F1(ncols,k,1.0/Y(k,k));
#ifdef verbose
F1.Print("F1");
#endif
Y= F1*Y;
S= F1*S;
#endif

}

/*:9*/
#line 150 "gauss.w"

#ifdef verbose
Y.Print("(4) Y");S.Print("(4) S");
#endif
/*10:*/
#line 227 "gauss.w"

{
for(int i= 0;i<nrows;i++)
{
if(i==k)
continue;

Real v= Y(i,k);

#ifdef efficiency_than_math

for(int j= 0;j<ncols;j++)
{
Y(i,j)-= v*Y(k,j);
S(i,j)-= v*S(k,j);
}

#else

FundOpeMatrixTypeIII F3(ncols,i,k,-v);

#ifdef verbose
F3.Print("F3");
#endif
Y= F3*Y;
S= F3*S;

#endif

}
}

/*:10*/
#line 154 "gauss.w"

#ifdef verbose
Y.Print("(5) Y");S.Print("(5) S");
#endif
}

return S;
}

/*:6*/
#line 59 "gauss.w"


/*:4*/
