/*34:*/
#line 736 "libmatrix.w"

/*3:*/
#line 94 "libmatrix.w"

 /* 
 Linear Calculation Library 
 
 Copyright (C) 2006: M Kudo & A. Tanaka
 
 Ver 1.0: 29 Sept 2006
 - This file is automatically generated from libmatrix.w. 
   See libmatrix.w for understanding the original code. 
   There is also a document "libmatrix.tex" for readability. 
 */

/*:3*/
#line 737 "libmatrix.w"


/*:34*//*35:*/
#line 740 "libmatrix.w"

#include <iostream> 
#include "math.h"
#include "matrix.h"

/*:35*//*36:*/
#line 746 "libmatrix.w"


using namespace std;

/*:36*//*37:*/
#line 754 "libmatrix.w"

Real Matrix::equiv_eps= 0.0;

/*:37*//*38:*/
#line 760 "libmatrix.w"

int RandomMatrix::gseed= 6852;
int RandomMatrix::called_num= 0;
int RandomIntMatrix::gseed= 6852;
int RandomIntMatrix::called_num= 0;

/*:38*//*39:*/
#line 769 "libmatrix.w"

void Matrix::Init(int n,int m,Real initv)
{
#ifdef secure
if(n<=0||m<=0)
{
cerr<<"*E* Init size exception of ("<<n<<","<<m<<") for matrix"<<endl;
exit(0);
}
#endif
nrows= n;
ncols= m;
try{
elements= new Real[n*m];
}
catch(std::bad_alloc&)
{
cerr<<"*E* In Matrix:Init():"<<
"Sorry ! we cannot find enough memory space for the requested size "
<<n*m*sizeof(Real)<<" byte for "<<n<<" x "<<m<<" matrix."<<
endl;
exit(1);
}
#ifdef const_check
cerr<<"** Memory was allocated at "<<elements<<" for "<<nrows<<" x "<<ncols<<
" matrix"<<endl;
#endif

for(int i= 0;i<n*m;i++)
elements[i]= initv;
}

/*:39*//*40:*/
#line 802 "libmatrix.w"

Matrix::Matrix(int n,int m,Real initv)
{
#ifdef const_check
fprintf(stderr,"*Const. ");fflush(stderr);
#endif
Init(n,m,initv);
}

Matrix::~Matrix()
{
#ifdef const_check
fprintf(stderr,"*Deconst. ");fflush(stderr);
cerr<<"** Memory is going to be released at "<<
elements<<endl;
#endif
delete[]elements;
}

int Matrix::Release()
{
delete[]elements;
elements= new Real[1];
}

Matrix::Matrix(const Matrix&X)
{
#ifdef const_check
fprintf(stderr,"*Copy Const. with size %d x %d ",X.nrows,X.ncols);fflush(stderr);
#endif

Init(X.nrows,X.ncols);

#ifdef verbose
cerr<<"*C* Copy destination memory is at "<<elements<<endl;
#endif
for(int i= 0;i<X.nrows*X.ncols;i++)
elements[i]= X.elements[i];
}

/*:40*//*41:*/
#line 845 "libmatrix.w"


Matrix&Matrix::operator= (const Matrix&X)
{
#ifdef const_check
fprintf(stderr,"*= Const. ");fflush(stderr);
#endif
if(this!=&X)
{
delete[]elements;
Init(X.nrows,X.ncols);


for(int i= 0;i<X.nrows*X.ncols;i++)
elements[i]= X.elements[i];
}
return*this;
}

Matrix&Matrix::operator<<(const Matrix&X)

{
int flag= 0;
#ifdef secure
if(X.nrows*X.ncols> nrows*ncols)
{
fprintf(stderr,"In Matrix op. !=: cannot insert a larger matrix into a smaller matrix\n");
fprintf(stderr," From %d x %d -> %d x %d\n",X.nrows,X.ncols,nrows,ncols);
fflush(stderr);
exit(0);
}
if(X.nrows*X.ncols<nrows*ncols)
{
fprintf(stderr,"In Matrix op. !=: insert a smaller matrix into a larger matrix\n");
fprintf(stderr," From %d x %d -> %d x %d\n",X.nrows,X.ncols,nrows,ncols);
fprintf(stderr,"... fill out zero's for remaining elements.\n");
fflush(stderr);
flag= 1;
}
#endif

if(this!=&X)
{
#ifdef const_check
fprintf(stderr,"<< Const. ");fflush(stderr);
#endif   





for(int i= 0;i<X.nrows*X.ncols;i++)
elements[i]= X.elements[i];

if(flag)
for(int i= X.nrows*X.ncols;i<nrows*ncols;i++)
elements[i]= 0.0;
}
return*this;
}

/*:41*//*42:*/
#line 912 "libmatrix.w"

RowVector&RowVector::operator= (const Matrix&X)
{
#ifdef secure
if(X.nrows!=1)
{
fprintf(stderr,"*E* cannot insert matrix with multiple rows into a row vector.\n");
fflush(stderr);
exit(0);
}
#endif
if(this!=&X)
{
delete[]elements;
Init(1,X.ncols);


for(int i= 0;i<X.ncols;i++)
elements[i]= X.elements[i];
}
return*this;
}

ColumnVector&ColumnVector::operator= (const Matrix&X)
{
#ifdef secure
if(X.ncols!=1)
{
fprintf(stderr,"*E*cannot inset matrix with multiple columns into a column vector.\n");
fflush(stderr);
exit(0);
}
#endif
if(this!=&X)
{
delete[]elements;
Init(X.nrows,1);


for(int i= 0;i<X.nrows;i++)
elements[i]= X.elements[i];
}
return*this;
}


/*:42*//*43:*/
#line 978 "libmatrix.w"

Real Matrix::Ele(int i,int j)const
{
#ifdef secure
if(i<0||j<0||i>=nrows||j>=ncols)
{
cerr<<"*E* Index exception of ("<<i<<","<<j<<") for matrix size ("
<<nrows<<","<<ncols<<")"<<endl;
exit(0);
}
#endif
return elements[i*ncols+j];
}

int Matrix::Elenum(int i,int j)const
{
#ifdef secure
if(i<0||j<0||i>=nrows||j>=ncols)
{
cerr<<"*E* Index exception of ("<<i<<","<<j<<") for matrix size ("
<<nrows<<","<<ncols<<")"<<endl;
exit(0);
}
#endif
return i*ncols+j;
}

Real Matrix::Ele(int k)const
{
#ifdef secure
if(k<0||k>=(nrows*ncols))
{
cerr<<"*E* Index exception of ("<<k<<
") for matrix size "<<nrows*ncols<<endl;
exit(0);
}
#endif
return elements[k];
}
/*:43*//*44:*/
#line 1018 "libmatrix.w"


Real&Matrix::operator()(int i,int j)
{
#ifdef verbose

#endif
return elements[Elenum(i,j)];
}

Real&Matrix::operator()(int k)
{
#ifdef verbose

#endif
#ifdef secure
if(k<0||k>=(nrows*ncols))
{
cerr<<"*E* Index exception of ("<<k<<
") for matrix size "<<nrows*ncols<<endl;
exit(0);
}
#endif
return elements[k];
}

const Real Matrix::operator()(int i,int j)const
{
#ifdef verbose

#endif
return elements[Elenum(i,j)];
}

const Real Matrix::operator()(int k)const
{
#ifdef verbose

#endif
return elements[k];
}

/*:44*//*45:*/
#line 1063 "libmatrix.w"

const int Matrix::Nrows()const
{
return nrows;
}

const int Matrix::Ncols()const
{
return ncols;
}

void Matrix::ReSize(int n,int m)
{
delete[]elements;
Init(n,m);
}

/*:45*//*46:*/
#line 1087 "libmatrix.w"

Matrix Matrix::Cols(int st,int ed)
{
if(st> ed)
{
int tmp= ed;
ed= st;
st= tmp;
}

int ycols= ed-st+1;

Matrix Y(nrows,ycols);
int k= 0;
for(int j= st;j<=ed;j++,k++)
for(int i= 0;i<nrows;i++)
Y(i,k)= Ele(i,j);

return Y;

}

Matrix Matrix::Rows(int st,int ed)
{
if(st> ed)
{
int tmp= ed;
ed= st;
st= tmp;
}

int yrows= ed-st+1;

Matrix Y(yrows,ncols);
int k= 0;
for(int i= st;i<=ed;i++,k++)
for(int j= 0;j<ncols;j++)
Y(k,j)= Ele(i,j);

return Y;

}

Matrix Matrix::SubMatrix(int rst,int red,int cst,int ced)

{
if(rst> red)
{
int tmp= red;
red= rst;
rst= tmp;
}

if(cst> ced)
{
int tmp= ced;
ced= cst;
cst= tmp;
}

#ifdef secure
if(rst<0||cst<0||red>=nrows||ced>=ncols)
{
cerr<<"*E* size exception of ("<<rst<<","<<cst<<") -"<<
"("<<red<<","<<ced<<") region"<<endl;
exit(0);
}
#endif

int yrows= red-rst+1;
int ycols= ced-cst+1;

Matrix Y(yrows,ycols);



for(int i= rst,k= 0;i<=red;i++,k++)
for(int j= cst,l= 0;j<=ced;j++,l++)
Y(k,l)= Ele(i,j);

return Y;

}

/*:46*//*47:*/
#line 1172 "libmatrix.w"


void Matrix::SetCols(int st,int ed,const Matrix&X)
{

if(st> ed)
{
int tmp= ed;
ed= st;
st= tmp;
}

int w= ed-st+1;
int wM= X.Ncols();

#ifdef secure
if(w!=wM)
{
cerr<<"In SetCols: type mismatch "<<wM<<" cols of a matrix into "<<
w<<"columns"<<endl;
exit(0);
}
#endif

int ycols= ed-st+1;
int c= 0;
for(int j= st;j<=ed;j++,c++)
for(int i= 0;i<nrows;i++)
elements[Elenum(i,j)]= X(i,c);
}

void Matrix::SetRows(int st,int ed,const Matrix&X)
{

if(st> ed)
{
int tmp= ed;
ed= st;
st= tmp;
}

int w= ed-st+1;
int wM= X.Nrows();

#ifdef secure
if(w!=wM)
{
cerr<<"In SetCols: type mismatch "<<wM<<" rows of a matrix into "<<w<<
" rows"<<endl;
exit(0);
}
#endif

int ycols= ed-st+1;
int c= 0;
for(int j= st;j<=ed;j++,c++)
for(int i= 0;i<ncols;i++)
elements[Elenum(j,i)]= X(c,i);
}

void Matrix::SetSubMatrix(int rst,int red,int cst,int ced,const Matrix&X)

{

if(rst> red)
{
int tmp= red;
red= rst;
rst= tmp;
}

if(cst> ced)
{
int tmp= ced;
ced= cst;
cst= tmp;
}

#ifdef secure
if(rst<0||cst<0||red>=nrows||ced>=ncols)
{
cerr<<"*E* size exception of ("<<rst<<","<<cst<<") -"<<
"("<<red<<","<<ced<<") region"<<endl;
exit(0);
}
#endif

int yrows= red-rst+1;
int ycols= ced-cst+1;

for(int j= rst,k= 0;j<=red;j++,k++)
for(int i= cst,l= 0;i<=ced;i++,l++)
elements[Elenum(j,i)]= X(k,l);
}

/*:47*//*48:*/
#line 1268 "libmatrix.w"


Matrix Matrix::t()const
{
Matrix Y(ncols,nrows);
for(int i= 0;i<nrows;i++)
for(int j= 0;j<ncols;j++)
Y(j,i)= Ele(i,j);
return Y;
}

Real Matrix::Tr()const
{
Real t= 0.0;
for(int i= 0;i<nrows;i++)
t+= Ele(i,i);
return t;
}

Real Matrix::Norm()const
{
Real t= 0.0;
for(int i= 0;i<nrows;i++)
for(int j= 0;j<ncols;j++)
t+= Ele(i,j)*Ele(i,j);
return sqrt(t);
}

Real Matrix::Norm2()const
{
Real t= 0.0;
for(int i= 0;i<nrows;i++)
for(int j= 0;j<ncols;j++)
t+= Ele(i,j)*Ele(i,j);
return t;
}

/*:48*//*49:*/
#line 1311 "libmatrix.w"

Matrix Matrix::i()const
{
Matrix Y(nrows,ncols);
Y= GaussJordan(*this);
return Y;
}

/*:49*//*50:*/
#line 1320 "libmatrix.w"

Matrix Matrix::operator+(const Matrix&X)const
{
#ifdef secure
if(nrows!=X.nrows||ncols!=X.ncols)
{
cerr<<"*E* Addition mismatch in A+B "<<
" A: "<<nrows<<" x "<<ncols<<
" B: "<<X.nrows<<" x "<<X.ncols<<endl;
exit(0);
}
#endif
Matrix Y(nrows,ncols);

for(int i= 0;i<Y.Nrows();i++)
for(int j= 0;j<Y.Ncols();j++)
Y(i,j)= Ele(i,j)+X.Ele(i,j);

return Y;
}

Matrix Matrix::operator+= (const Matrix&X)
{
#ifdef secure
if(nrows!=X.nrows||ncols!=X.ncols)
{
cerr<<"*E* Addition mismatch in A+B "<<
" A: "<<nrows<<" x "<<ncols<<
" B: "<<X.nrows<<" x "<<X.ncols<<endl;
exit(0);
}
#endif
for(int i= 0;i<nrows;i++)
for(int j= 0;j<ncols;j++)
elements[i*ncols+j]+= X.Ele(i,j);

return*this;
}


Matrix Matrix::operator-(const Matrix&X)const
{
#ifdef secure
if(nrows!=X.nrows||ncols!=X.ncols)
{
cerr<<"*E* Addition mismatch in A-B "<<
" A: "<<nrows<<" x "<<ncols<<
" B: "<<X.nrows<<" x "<<X.ncols<<endl;
exit(0);
}
#endif
Matrix Y(nrows,ncols);

for(int i= 0;i<Y.Nrows();i++)
for(int j= 0;j<Y.Ncols();j++)
Y(i,j)= Ele(i,j)-X.Ele(i,j);

return Y;
}

Matrix Matrix::operator-= (const Matrix&X)
{
#ifdef secure
if(nrows!=X.nrows||ncols!=X.ncols)
{
cerr<<"*E* Addition mismatch in A-B "<<
" A: "<<nrows<<" x "<<ncols<<
" B: "<<X.nrows<<" x "<<X.ncols<<endl;
exit(0);
}
#endif
for(int i= 0;i<nrows;i++)
for(int j= 0;j<ncols;j++)
elements[i*ncols+j]-= X.Ele(i,j);

return*this;
}


Matrix Matrix::operator*(const Matrix&X)const
{
#ifdef secure
if(ncols!=X.nrows)
{
cerr<<"*E* Multiplication mismatch in A*B : A.ncols "<<ncols<<" but"<<" B.nrows "
<<X.nrows<<endl;
exit(0);
}
#endif

Matrix Y(nrows,X.ncols);
Real v;
for(int i= 0;i<Y.Nrows();i++)
for(int j= 0;j<Y.Ncols();j++)
{
v= 0.0;
for(int k= 0;k<ncols;k++)
v+= Ele(i,k)*X.Ele(k,j);
Y(i,j)= v;
}

return Y;
}

Matrix Matrix::operator*= (const Matrix&X)
{
*this= *this*X;
return*this;
}

Matrix Matrix::operator*(const Real&x)const
{
Matrix Y(nrows,ncols);

for(int i= 0;i<Y.Nrows();i++)
for(int j= 0;j<Y.Ncols();j++)
Y(i,j)= x*Ele(i,j);

return Y;
}

Matrix Matrix::operator*= (const Real&x)
{
*this= *this*x;
return*this;
}

Matrix Matrix::operator/(const Real&x)const
{
Matrix Y(nrows,ncols);

for(int i= 0;i<Y.Nrows();i++)
for(int j= 0;j<Y.Ncols();j++)
Y(i,j)= Ele(i,j)/x;

return Y;
}

Matrix Matrix::operator/= (const Real&x)
{
*this= *this/x;
return*this;
}


Matrix operator*(const Real&x,const Matrix&X)
{
Matrix Y= X;
Y*= x;
return Y;
}

/*:50*//*51:*/
#line 1479 "libmatrix.w"

void Matrix::Eigen(Matrix&Eval,Matrix&Evec)
{
Jacobi(*this,Eval,Evec);
}

Real Matrix::Eigen(Matrix&Eval,Matrix&Evec,int&rank)
{
Real ratio= (-1);
MaximumEigenMethod(*this,Eval,Evec,rank,ratio);
return ratio;
}

int Matrix::Eigen(Matrix&Eval,Matrix&Evec,Real&ratio)
{
int rank= (-1);
MaximumEigenMethod(*this,Eval,Evec,rank,ratio);
return rank;
}

/*:51*//*52:*/
#line 1502 "libmatrix.w"


const int Maximum_Element_Num= 500000;

int Matrix::Read(FILE*fp,File_type ftype)
{
cout<<"File type ="<<ftype<<endl;
switch(ftype)
{
case PGM:
cout<<"PGM File"<<endl;

{
char s1[256],s2[256],dummy[256];
char*d= dummy;
int value[4];
int type;
int x_size= 0,y_size= 0;
int v_max= 0;
int rv_flag= 0;
Real rv_min= 0.0,rv_max= 255.0;
char*word;
int cc= 0;

while(1)
{
d= dummy;
for(;;)
{
fgets(d,256,fp);
if(d[0]!='#')
break;
else
{
d+= 2;
if(*d=='v')
{
d-= 2;
sscanf(d,"# v_min = %lf v_max = %lf \n",&rv_min,&rv_max);
if(rv_min!=0.0&&rv_max!=255.0)
{
rv_flag= 1;
cout<<"v_min = "<<rv_min<<" v_max = "<<rv_max<<endl;
}
}
}

}

while(word= strtok(d," "))
{

if(cc==0)
{
value[0]= (strncmp(word,"P5",2)==0)?5:2;
if(strncmp(word,"P5",2)!=0&&strncmp(word,"P2",2)!=0)
{
fprintf(stderr,"File Type is not pgm( P2 or P5)\n");
exit(0);
}
cc++;
}
else
value[cc++]= atoi(word);
d= 0;
}

if(cc>=4)
{
type= value[0];
x_size= value[1];
y_size= value[2];
v_max= value[3];
break;
}
}

printf("- Header Info :type %d x_size=%d y_size=%d v_max=%d rv_min=%f rv_max=%f\n",
type,x_size,y_size,v_max,rv_min,rv_max);

ReSize(y_size,x_size);

int c= 0;
int n= x_size*y_size;

int dum;

double rv_w= rv_max-rv_min;
double isc= rv_w/double(255);

if(type==2)
{
while(n--)
{
fscanf(fp," %d",&dum);
if(rv_flag)
elements[c++]= dum*isc+rv_min;
else
elements[c++]= dum;
}
}

if(type==5)
{
while(n--)
{
unsigned char ch;
if(fread((char*)&ch,1,1,fp)!=1)
{
fprintf(stderr,"*E*(Region::PGM_Read) Cannot read Byte %d\n",c);
exit(0);
}

if(rv_flag)
elements[c++]= Real(ch)*isc+rv_min;
else
elements[c++]= Real(ch);
}
}

}
break;

case Raw:
default:
cout<<"Raw ASCII File"<<endl;

int nw;
const int Max_Line= 2048;
char line[Max_Line];
char word[Max_Line][50];




ColumnVector Tmp(Maximum_Element_Num);



int n= 0;
int m= 0;
int c= 0;
while((nw= getwords_from_line(fp,line,word,Max_Line))!=0)
{
if(n==0)
m= nw;

for(int i= 0;i<m;i++)
Tmp.elements[c++]= atof(word[i]);

n++;

if(n>=Maximum_Element_Num)
{
cerr<<"*E* cannot read such a large matrix. The allowed size is at most "
<<Maximum_Element_Num<<"in rows by columns"<<endl;
exit(0);
}

}
cout<<"Nrows = "<<n<<" Ncols= "<<m<<endl;

ReSize(n,m);

c= 0;
for(int i= 0;i<n;i++)
for(int j= 0;j<m;j++)
{
elements[c]= Tmp.elements[c];
c++;

}

break;
}

}
int Matrix::Write(FILE*fp,File_type ftype)
{

switch(ftype)
{
case PGM:
cout<<"PGM File of "<<nrows<<" x "<<ncols<<endl;



{
const int flag= 0;
double v_min= 1.e10;
double v_max= -1.e10;
int nsize= ncols*nrows;
{
for(int j= 0;j<nsize;j++)
{
double p= Ele(j);
if(p> v_max){v_max= p;}
if(p<v_min){v_min= p;}
}
}

int cc= 0;
double v_w= v_max-v_min;

if(flag==2)
{
v_max= 255.0;
v_min= 0.0;
v_w= 255.0;
}

double sc= double(255)/v_w;

int max= int(v_w);

if(flag)
fprintf(fp,"P2\n");
else
fprintf(fp,"P5\n");

fprintf(fp,"# For pgm file \n");
fprintf(fp,"# v_min = %f v_max = %f \n",v_min,v_max);
fprintf(fp,"# Generated by Matrix Library matrix.a\n");
fprintf(fp,"%d %d\n",ncols,nrows);

fprintf(fp,"255\n");

for(int j,i= 0;i<nrows;i++)
for(j= 0;j<ncols;j++)
{
if(flag)
{
if((cc%16)==0)fprintf(fp,"\n");
cc++;
fprintf(fp,"%5d ",int((Ele(i,j)-v_min)*sc));
}
else
{
char c= int((Ele(i,j)-v_min)*sc);
fwrite(&c,1,1,fp);
}
}

}
break;

case Raw:
default:
cout<<"Raw ASCII File"<<endl;
for(int i= 0;i<nrows;i++)
{
for(int j= 0;j<ncols;j++)
fprintf(fp,"%f ",Ele(i,j));
if(ftype!=Raw_Straight)
fprintf(fp,"\n");
}
if(ftype==Raw_Straight)
fprintf(fp,"\n");

break;
}
}

/*:52*//*53:*/
#line 1768 "libmatrix.w"

bool Matrix::operator==(Matrix&B)

{
if(nrows!=B.nrows||ncols!=B.ncols)
return false;

#ifdef verbose
cout<<"In Matrix::op == : Norm of A-B = "<<(*this-B).Norm()<<
" for eps "<<equiv_eps<<endl;
#endif

if((*this-B).Norm()> equiv_eps)
return false;

return true;
}

bool Matrix::operator!=(Matrix&B)
{
return(*this==B)?false:true;
}

/*:53*//*54:*/
#line 1792 "libmatrix.w"


void Matrix::Print(char*s)const
{
printf("%s (nrows=%d, ncols=%d)\n",s,nrows,ncols);fflush(stdout);
for(int i= 0;i<nrows;i++)
{
for(int j= 0;j<ncols;j++)
printf("%f ",Ele(i,j));
printf("\n");
}
fflush(stdout);
}

void Matrix::Output(FILE*fp= stdout,char*s)const
{
for(int i= 0;i<nrows;i++)
{
for(int j= 0;j<ncols;j++)
fprintf(fp,"%f ",Ele(i,j));
fprintf(fp,"\n");
}
fflush(fp);
}


/*:54*//*55:*/
#line 1835 "libmatrix.w"


int getwords_from_line(FILE*fp,char*line,char word[][50],int max)
{

char*p= line,*q;
int i= 0,c;

if(fgets(line,max,fp)==NULL)
return 0;

if(strlen(line)>=(max-1))
{
cerr<<"*Error in getwords_from_line* input line is too long ... over "<<max<<"\n";
exit(0);
}

if(line[0]=='#'||line[0]=='%')
return(-1);
q= word[i];
while(1)
{
while(isspace(*p))p++;
c= 0;
while(!isspace(*p)&&*p!='\0'){*q++= *p++;c++;}
*q++= '\0';

if(c==0)
break;

q= word[++i];

}

return i;
}

/*:55*//*62:*/
#line 2174 "libmatrix.w"


Real Matrix::Mean()
{
Real sum= Ele(0);
for(int i= 1;i<nrows*ncols;i++)
sum= (i*sum+Ele(i))/Real(i+1);

return sum;
}

Matrix Matrix::MeanVec()
{
Matrix M(nrows,1,0.0);
M= Cols(0,0);
for(int i= 1;i<ncols;i++)
M= (M*i+Cols(i,i))/Real(i+1);

return M;
}

Real Matrix::Variance(Real mean)
{
Real sum= (Ele(0)-mean)*(Ele(0)-mean);
for(int i= 1;i<nrows*ncols;i++)
sum= (i*sum+(Ele(i)-mean)*(Ele(i)-mean))/Real(i+1);
return sum;
}

Matrix Matrix::VarianceMatrix(Matrix&M)
{
Matrix Cov(nrows,nrows);
Cov= (Cols(0,0)-M)*(Cols(0,0)-M).t();

for(int i= 1;i<ncols;i++)
Cov= (Cov*i+(Cols(i,i)-M)*(Cols(i,i)-M).t())/Real(i+1);

return Cov;

}/*:62*/
