/*4:*/
#line 107 "libmatrix.w"

/*3:*/
#line 94 "libmatrix.w"

 /* 
 Linear Calculation Library 
 
 Copyright (C) 2006: M Kudo & A. Tanaka
 
 Ver 1.0: 29 Sept 2006
 - This file is automatically generated from libmatrix.w. 
   See libmatrix.w for understanding the original code. 
   There is also a document "libmatrix.tex" for readability. 
 */

/*:3*/
#line 108 "libmatrix.w"


/*:4*//*5:*/
#line 115 "libmatrix.w"

#ifndef matrix_h
#define matrix_h

/*:5*//*6:*/
#line 122 "libmatrix.w"

typedef double Real;


/*:6*//*7:*/
#line 127 "libmatrix.w"

enum YESNO{NO= 0,YES= 1};


/*:7*//*8:*/
#line 146 "libmatrix.w"


//#define verbose  // if you need more info.
//#define const_check  // display when constructor or deconstructor is evoked.
#define secure   // if you need slow but secure version. 
#define efficiency_than_math // put priority on efficiency rather 
                            // than mathemitic understandability 

/*:8*//*9:*/
#line 156 "libmatrix.w"


using namespace std;

/*:9*//*10:*/
#line 163 "libmatrix.w"

enum File_type{Raw,Raw_Straight,PGM};

/*:10*//*11:*/
#line 174 "libmatrix.w"


class Matrix
{
/*12:*/
#line 207 "libmatrix.w"

friend class IdentityMatrix;
friend class ColumnVector;
friend class RowVector;
friend class RandomMatrix;
friend class RandomIntMatrix;
friend class FundOpeMatrixTypeI;
friend class FundOpeMatrixTypeII;
friend class FundOpeMatrixTypeIII;

/*:12*/
#line 178 "libmatrix.w"


public:
/*13:*/
#line 241 "libmatrix.w"


Matrix(int= 1,int= 1,Real= 0.0);

~Matrix();
Matrix(const Matrix&X);
int Release();

/*:13*//*14:*/
#line 260 "libmatrix.w"

Matrix&operator= (const Matrix&X);
Matrix&operator<<(const Matrix&X);


/*:14*/
#line 181 "libmatrix.w"

/*15:*/
#line 293 "libmatrix.w"



Real&operator()(int,int);

Real&operator()(int);

const Real operator()(int,int)const;

const Real operator()(int)const;

/*:15*/
#line 182 "libmatrix.w"

/*16:*/
#line 318 "libmatrix.w"


const int Nrows()const;
const int Ncols()const;
void ReSize(int,int);

/*:16*/
#line 183 "libmatrix.w"

/*17:*/
#line 340 "libmatrix.w"


Matrix Cols(int,int);
Matrix Rows(int,int);
Matrix SubMatrix(int,int,int,int);
void SetCols(int,int,const Matrix&);
void SetRows(int,int,const Matrix&);
void SetSubMatrix(int,int,int,int,const Matrix&);

/*:17*/
#line 184 "libmatrix.w"

/*18:*/
#line 359 "libmatrix.w"


Matrix t()const;
Matrix i()const;
Real Tr()const;
Real Rank()const;
Real Norm()const;
Real Norm2()const;

/*:18*/
#line 185 "libmatrix.w"

/*19:*/
#line 387 "libmatrix.w"


Matrix operator+(const Matrix&X)const;
Matrix operator-(const Matrix&X)const;
Matrix operator*(const Matrix&X)const;

Matrix operator+= (const Matrix&X);
Matrix operator-= (const Matrix&X);
Matrix operator*= (const Matrix&X);

Matrix operator*(const Real&x)const;
Matrix operator/(const Real&x)const;
Matrix operator*= (const Real&x);
Matrix operator/= (const Real&x);

/*:19*/
#line 186 "libmatrix.w"

/*20:*/
#line 434 "libmatrix.w"


void Eigen(Matrix&,Matrix&);
Real Eigen(Matrix&,Matrix&,int&);

int Eigen(Matrix&,Matrix&,Real&);


/*:20*/
#line 187 "libmatrix.w"

/*21:*/
#line 459 "libmatrix.w"


int Read(FILE*,File_type= Raw);
int Write(FILE*,File_type= Raw);

/*:21*/
#line 188 "libmatrix.w"

/*22:*/
#line 485 "libmatrix.w"


void SetEquivValue(Real v){equiv_eps= v;}
Real EquivValue(){return equiv_eps;}
bool operator==(Matrix&);
bool operator!=(Matrix&);
static Real equiv_eps;


/*:22*/
#line 189 "libmatrix.w"

/*23:*/
#line 501 "libmatrix.w"


void Print(char* = "Matrix")const;
void Output(FILE*fp,char* = "Matrix")const;

/*:23*/
#line 190 "libmatrix.w"

/*61:*/
#line 2167 "libmatrix.w"

Real Mean();
Matrix MeanVec();
Real Variance(Real mean);
Matrix VarianceMatrix(Matrix&);

/*:61*/
#line 191 "libmatrix.w"


private:
void Init(int,int,Real= 0.0);
Real Ele(int,int)const;
Real Ele(int)const;
int Elenum(int,int)const;
int nrows;
int ncols;
Real*elements;
};
/*24:*/
#line 508 "libmatrix.w"

Matrix operator*(const Real&x,const Matrix&);

/*:24*//*25:*/
#line 528 "libmatrix.w"

class ColumnVector:public Matrix
{
public:
ColumnVector(int n,Real initv= 0.0):Matrix(n,1,initv)
{

}
ColumnVector&operator= (const Matrix&X);

};

class RowVector:public Matrix
{
public:
RowVector(int n,Real initv= 0.0):Matrix(1,n,initv)
{

}
RowVector&operator= (const Matrix&X);
};

class IdentityMatrix:public Matrix
{
public:
IdentityMatrix(int n):Matrix(n,n)
{

for(int i= 0;i<n;i++)
elements[i*n+i]= 1.0;
}
};


/*:25*//*26:*/
#line 573 "libmatrix.w"

class RandomMatrix:public Matrix
{
public:
RandomMatrix(int n,int m,unsigned int seed= 0):Matrix(n,m)
{
if(seed!=0)
{
gseed= seed;
srand(gseed);
}

called_num++;

for(int i= 0;i<n;i++)
for(int j= 0;j<m;j++)
elements[i*m+j]= 1.0*rand()/(RAND_MAX+1.0);
}
private:
static int gseed;
static int called_num;
};

class RandomIntMatrix:public Matrix
{
public:
RandomIntMatrix(int n,int m,int max= 10,int sgn= 1,unsigned int seed= 0):Matrix(n,m)
{
if(seed!=0)
{
gseed= seed;
srand(gseed);
}

called_num++;

srand(seed);
if(sgn>=0)
{
for(int i= 0;i<n;i++)
for(int j= 0;j<m;j++)
elements[i*m+j]= int(Real(max)*(rand()/(RAND_MAX+1.0)));
}
else
{
for(int i= 0;i<n;i++)
for(int j= 0;j<m;j++)
elements[i*m+j]= int(2.0*Real(max)*(rand()/(RAND_MAX+1.0)-0.5));
}
}
private:
static int gseed;
static int called_num;
};

/*:26*//*27:*/
#line 640 "libmatrix.w"













/*:27*//*28:*/
#line 656 "libmatrix.w"


class FundOpeMatrixTypeI:public Matrix
{
public:
FundOpeMatrixTypeI(int n,int k,Real lambda):Matrix(n,n)
{
#ifdef verbose
cerr<<"Const of fundamental operation matrix type 1 with lambda"<<endl;
#endif
for(int i= 0;i<n;i++)
elements[i*n+i]= 1.0;

elements[k*n+k]= lambda;
}
};

/*:28*//*29:*/
#line 675 "libmatrix.w"


class FundOpeMatrixTypeII:public Matrix
{
public:
FundOpeMatrixTypeII(int n,int i,int j):Matrix(n,n)
{
#ifdef verbose
cerr<<"Const of fund. operation matrix type 2 : exchange rows (or columns) i and j "<<endl;
#endif
for(int k= 0;k<n;k++)
elements[k*n+k]= 1.0;

elements[i*n+i]= 0.0;
elements[j*n+j]= 0.0;
elements[i*n+j]= 1.0;
elements[j*n+i]= 1.0;
}
};

/*:29*//*30:*/
#line 697 "libmatrix.w"


class FundOpeMatrixTypeIII:public Matrix
{
public:
FundOpeMatrixTypeIII(int n,int i,int j,Real lambda):Matrix(n,n)
{
#ifdef verbose
cerr<<"Const of fund. ope. matrix type 3 : plus row (or column) i with lambda*j "
<<endl;
#endif
for(int k= 0;k<n;k++)
elements[k*n+k]= 1.0;

elements[i*n+j]= lambda;
}
};


/*:30*//*31:*/
#line 717 "libmatrix.w"


Matrix GaussJordan(const Matrix&X);
int Jacobi(const Matrix&X,Matrix&Z,Matrix&U);

int getwords_from_line(FILE*fp,char*line,char word[][50],int max);
void MaximumEigenMethod(const Matrix&X,Matrix&Z,Matrix&U,int&,Real&);


/*:31*//*32:*/
#line 727 "libmatrix.w"

#endif

/*:32*/
#line 202 "libmatrix.w"


/*:11*/
